require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::StatusesController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:user) {create(:user)}
  let!(:housing_offer){create(:housing_offer)}
  #TESTS BASIQUES D'API

  describe 'GET /statuses' do
    let!(:status1) {create(:status, object_id: housing_offer.id.to_s, status_label: 'offer_status')}
    let!(:status3) {create(:status, object_id: housing_offer.id.to_s, status_label: 'offer_status')}
    let!(:status2) {create(:status, object_id: housing_offer.id.to_s, status_label: 'offer_status')}

    describe 'GET /statuses' do
      it 'when admin' do
        authenticated_header(request, responsible)
        get 'index'
        expect(response).to have_http_status(200)
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end

      it 'unauthorized normal user' do 
        authenticated_header(request, user)
        get 'index'
        expect(response).to have_http_status(401)
      end 
    end

    describe 'GET /statuses with operators' do
      let!(:status1) {create(:status, motive: 'acceuillant', object_id: housing_offer.id.to_s, status_label: 'offer_status')}
      let!(:status3) {create(:status, motive: 'Super', object_id: housing_offer.id.to_s, status_label: 'offer_status')}
      let!(:status2) {create(:status, motive: 'bel appart', object_id: housing_offer.id.to_s, status_label: 'offer_status')}

      describe 'with $in' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {motive: {'$in': ['acceuillant', 'flamand']}}
        end
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end
      describe 'with $ne' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {motive: {'$ne': 'acceuillant'}}
        end
        it 'returns sorted results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(2)
        end
      end
      describe 'with $regex' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {motive: {'$regex': /^S/}}
        end
        it 'returns sorted results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end
    end
  end

  describe 'GET /statuses/:id' do
    describe 'verif it work' do
      let!(:status1) {create(:status, motive: 'acceuillant', object_id: housing_offer.id.to_s, status_label: 'offer_status')}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: status1.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
      it 'not have weird object_id' do 
        expect(json['data']['attributes']['object_id']).to eq(housing_offer.id.to_s)
      end 
    end
    describe 'verif it work not work with a non authorized user' do
      let!(:status1) {create(:status, object_id: housing_offer.id.to_s, status_label: 'offer_status')}

      before :each do
        authenticated_header(request, user)
        get 'show', {id: status1.id}
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(401)
      end
    end
  end

  describe 'GET /statuses/applications_history' do
    let!(:offer) { create(:housing_offer)}
    let!(:demandeur) { create :user, roles: ['demandeur'] }
    let!(:app) {create(:applicant, user: demandeur)}
    let!(:hr) { create(:housing_request, main_applicant: app, user: demandeur, is_active: true)}
    let!(:application) { create(:housing_application, housing_request: hr, housing_offer: offer) }
    let!(:rand_application) { create(:active_application) }
    let!(:status_1) { create :status, object_id: application.id, status_label: 'status' , title: 'ANNULEE' }
    let!(:status_2) { create :status, object_id: application.id, status_label: 'status' , title: 'INSCRIPTION CAL' }
    let!(:status_3) { create :status, object_id: application.id, status_label: 'status' , title: 'TRANSMIS BAILLEUR' }
    let!(:statuses) { create_list :status, 4, object_id: rand_application.id, status_label: 'status' , title: 'NON RETENUE' }

    context 'when demandeur' do
      let(:expected_ids) { json['data'].map { |h| h['id'] } }

      before :each do
        status_1.set(updated_at: Time.zone.now - 3.days)
        status_2.set(updated_at: Time.zone.now + 5.days)

        authenticated_header(request, demandeur)
        get 'applications_history'
      end
      it 'succeeds' do
        expect(response).to have_http_status 200
      end

      it 'returns only the user applications_history' do
        expect(json['data'].length).to eq 3
        expect(expected_ids).to eq [status_1.id.to_s, status_2.id.to_s, status_3.id.to_s]
      end

      context 'when start_date' do
        it 'filters by updated_at' do
          get 'applications_history', { start_date: Time.zone.now - 2.days }
          expect(json['data'].length).to eq 2
          expect(expected_ids).to eq [status_2.id.to_s, status_3.id.to_s]
        end
      end

      context 'when end_date' do
        it 'filters by updated_at' do
          get 'applications_history', { end_date: Time.zone.now + 4.days }
          expect(json['data'].length).to eq 2
          expect(expected_ids).to eq [status_1.id.to_s, status_3.id.to_s]
        end
      end

      context 'when both dates' do
        it 'filters by updated_at' do
          get 'applications_history', { start_date: Time.zone.now - 2.days, end_date: Time.zone.now + 4.days }
          expect(json['data'].length).to eq 1
          expect(expected_ids).to eq [status_3.id.to_s]
        end
      end
    end

    context 'when other role' do
      before :each do
        authenticated_header(request, [responsible, user].sample)
        get 'applications_history'
      end
      it 'returns unauthorized error' do
        expect(response).to have_http_status 401
      end
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::StatusesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_STATUSES'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }
  let!(:demandeur) { create(:user, auth_roles: ['demandeur']) }

  path '/api/dmo/statuses' do
    get 'Retrieves all statuses' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::StatusesController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'statuses found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        run_test!
      end
    end
  end

  path '/api/dmo/statuses/{id}' do
    get 'Retrieves a single status by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 
      
      response '200', 'status found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:status) {create(:status)}
        let(:id) {status.id}

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {status}
        end
      end
    end
  end

  path '/api/dmo/statuses/applications_history' do
    get 'Retrieves all statuses of an applications' do
      tags tags
      produces 'application/json'

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'statuses found' do
        let(:Authorization) { "Bearer #{token_for(demandeur)}" }
        let!(:statuses) { create_list(:status, 5) }

        run_test!
      end
    end
  end

end
