# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'

dir_path = File.dirname(__FILE__)
schema_path = [dir_path, 'FluxPatrimoine.xsd'].join('/')
require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::AddOfferTask, type: :helper do
  let!(:xsd) { Nokogiri::XML::Schema(File.read(schema_path)) }

  it 'should load a valid XSD' do
    expect(xsd).to be_present
    expect(xsd.class).to be Nokogiri::XML::Schema
  end

  context 'with simple housing_offer' do
    let!(:lessor1) do
      create(:lessor,
             title: 'MON ENTREPRISE',
             siren: '716820410',
             siret: '71682041000026',
             )
    end
    THRESHOLD_MAP = {
      'Paris et alentours' => {
        'thresholds_list' => %w[
          12673
          14679
        ],
        'another_person' => '6723'
      }
    }.freeze
    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:threshold2) { create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'PLA LM', codes: ['PLM']) }
    let!(:agency1) { create(:entity, title: 'MON AGENCE', siren: '123456789', lessor: lessor1) }
    let!(:housing_offer1) do
      create(
        :housing_offer,
        lessor: lessor1,
        rent_with_charges:  350.0,
        rent_amount: 300.0,
        rental_charges: 50.0,
        agency_id: "ALSSG-145",
        als_siren: "824541148",
        is_mad_updated_at: Date.current,
        availability_date: Date.current,
        designation_date_end: Date.current,
        has_gardian: nil,
        threshold_type: "PLU",
        external_ref: "5Ticket21667",
        description: "Appart idéal",
        address: "2 rue Bobigny",
        address_complements: nil,
        communicate_exact_address: nil,
        postal_code: "93000",
        virtual_visit: nil,
        furnished_lodging: nil,
        terraces_or_balconies: nil,
        already_inhabited: nil,
        residence_code: "ALSOP-237",
        residence_title: "La vie ",
        rpls_id: nil,
        construction_type: "COL",
        kind: "APT",
        typology: "T1",
        country: "France",
        latitude: 48.9066938,
        longitude: 2.4492356,
        coordinates: [2.4492356, 48.9066938],
        als_id: "9a44189b3223f37fb1",
        is_from_form: true,
        city: 'Bobigny',
        reference_mad: 'blabla',
        lessor_orientations: ['PDALO', 'PPAAC'],
        lessor_code: 'TEST'
      )
    end
    let!(:picture0) { create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id) }
    let!(:picture1) { create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id) }

    let!(:add_offer_task1) do
      build(
        :add_offer_task,
        options: { housing_offer_id: housing_offer1.id }
      )
    end

    describe 'using to_xml' do
      it 'should return a valid xml' do
        add_offer_task1.validate_xml
        expect(housing_offer1.reload.last_xml_validation_report).to eq('[]')
        expect(xsd.validate(Nokogiri::XML(add_offer_task1.to_xml)).length).to eq(0)

        VCR.use_cassette("flux5", :match_requests_on => [:method, :uri, :path]) do
          add_offer_task1.execute
          last_job = enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.last
          last_job[:job].perform_now(*deserialize_argument(last_job[:args]))
        end

        expect(housing_offer1.reload.last_send_has_error).to eq(false)
        expect(Hermes::XmlSopraError.count).to eq(0)
      end

      it 'throw 503 error' do 
        VCR.use_cassette("flux5_unavailable", :match_requests_on => [:method, :uri, :path]) do
          add_offer_task1.execute
          last_job = enqueued_jobs.select{|job| job[:job] == ExecuteSopraTaskJob }.last
          last_job[:job].perform_now(*deserialize_argument(last_job[:args]))
        end

        expect(housing_offer1.reload.last_send_has_error).to eq(true)
        expect(Hermes::XmlSopraError.count).to eq(1)
        xml_error = Hermes::XmlSopraError.last
        expect(xml_error.code).to eq("503")
        expect(xml_error.error_content).to include("RestClient::ServiceUnavailable ---->")
      end 
    end


    describe 'check xml' do 
      it 'reference_mad and lessor_orientations' do 
        so = Hermes::BoClient::AddOfferTask.new({:'housing_offer_id'=> housing_offer1.id})
        xml = so.to_xml
        parsed_xml = Hash.from_xml(xml.gsub("\n", ""))
        expect(parsed_xml['patrimoine']['bien']['reference_MAD']).to eq(housing_offer1.reference_mad)
        expect(parsed_xml['patrimoine']['bien']['orientations_bailleur']['orientation']).to eq(housing_offer1.lessor_orientations)
      end 

      it 'check code_territoire' do 
        so = Hermes::BoClient::AddOfferTask.new({:'housing_offer_id'=> housing_offer1.id})
        xml = so.to_xml
        parsed_xml = Hash.from_xml(xml.gsub("\n", ""))
        expect(parsed_xml['patrimoine']['bien']['code_territoire']).to eq("TEST")
      end 
    end 
  end
end
