require 'rails_helper'

RSpec.describe BatchTasks::Base, type: :service do
  let(:tagset_id) { 'tagset_test' }

  shared_examples 'log file operations' do |log_type|
    let(:log_name) { "test_log.#{log_type}" }
    let(:subject) { described_class.new(log_name) }
    
    after do
      File.delete(log_name) if File.exist?(log_name)
    end
    
    describe '#create_log_doc' do
      before do
        File.open(log_name, 'w') { |file| file.write('Test content') }
      end

      it 'reads and assigns file content to document blob' do
        doc = subject.create_log_doc(tagset_id)
        expect(doc.data).to include('Test content')
        expect(doc.tagset_ids).to include(tagset_id)
      end

      it 'deletes the log file after processing' do
        subject.create_log_doc(tagset_id)
        expect(File.exist?(log_name)).to be_falsey
      end
    end

    describe '#add_to_log_file' do
      let(:line) { log_type == 'csv' ? ['simple', 'data'] : 'simple data' }

      it 'appends the line to log_lines' do
        subject.add_to_log_file(line)
        expect(subject.log_lines).to include(line)
      end

      context 'when buffer is full or forced to flush' do
        it 'writes to the log file and clears the buffer' do
          1000.times { subject.add_to_log_file(line) }
          expect(File).to exist(log_name)
          expect(subject.log_lines).to be_empty
          content = File.read(log_name)
          expected_line = log_type == 'csv' ? line.join(';') + "\n" : line + "\n"
          expect(content).to eq(expected_line * 1000)
        end
      end
    end
  end

  context 'works with log files' do 
    include_examples('log file operations', 'log')
  end 

  context 'works with csv files' do
    include_examples('log file operations', 'csv')
  end 
end